c "/ngslib/source/Geoid/Flib/SCCS/s.interg.f

      subroutine interg(xlat,xlon,lin,kk, val)
ccccccccccccccccccccccccccccccccccccccccccccccccccc
c Interpolates a value (= val) from the kkth geoid grid, 
c using 6x6 spline, 4x4 spline, or biquadratic interpolation.
c As the gridded data are all direct-access files,
c only the nearest points (max = 36 real*4 values)
c are read into RAM for each point's interpolation.
c 
c The size/spacing/etc of the data file's grid
c are defined by the common statement, and
c the variable "kk"
c Caveats:
c   1) It is assumed that xlat/xlon fall in the region of the kth grid
c   in - xlat  : latitude  position, units = 
c   in - xlon  : longitude position, units = 
c   in - lin   : vector of file unit numbers, pointing to grid files
c   in - kk    : file unit number to be read for grid data
c  out - val   : geoid undulation, units as per data file
c 
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      implicit none
      character*90 SCCSID
      real*8     xlat
      real*8     xlon
      integer*4  lin(50)
      integer*4  kk
      real*4     val

      real*8     glamn(50),glamx(50)
      real*8     glomn(50),glomx(50)
      real*8     dla(50),dlo(50)

      real*4     f1
      real*4     xlatn
      real*4     xlonn

      integer*4  nla(50)
      integer*4  nlo(50)
      integer*4  ikind(50)
      integer    irown
      integer    icoln
      integer    irec

c variables and functions for handling endian problem
      real*4     frev4
      logical    reverse_bytes(50)
      common     /endian/reverse_bytes

      common     /grid1/ glamn,glamx,glomn,glomx
      common     /grid2/ dla,dlo,nla,nlo,ikind

      SCCSID="@(#)interg.f	1.7	2008/06/12 NGS"

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - A little check for safety's sake
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(xlat.gt.glamx(kk) .or. xlat.lt.glamn(kk) .or.
     *   xlon.lt.glomn(kk) .or. xlon.gt.glomx(kk))then
        stop 10000
      endif

c ----------------------------------------------------------
c Find the row/col of the nearest grid node to the lat/lon point
c This grid node is south and west from the lat/lon point
c ----------------------------------------------------------
      irown = int((xlat-glamn(kk)) / dla(kk)) + 1
      icoln = int((xlon-glomn(kk)) / dlo(kk)) + 1

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Find the latitude and longitude of the nearest
c - grid point
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      xlatn = glamn(kk) + (irown - 1)*dla(kk)
      xlonn = glomn(kk) + (icoln - 1)*dlo(kk)

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - First things first 
c - if we're sitting right on or near (0.36 arcsec)
c - a grid node, just assign the value and return
c - (1.0e-4 * 3600) = 0.36 arcsec
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if( (xlat-xlatn).le.1.0e-4 .and. (xlon-xlonn).le.1.0e-4)then
        irec = 11 + (irown - 1)*nlo(kk) + icoln 
        read(lin(kk),rec=irec)f1
c read big endian values and convert to little endian
        if(reverse_bytes(kk)) f1=frev4(f1)
        val = f1
        return
      endif

c ----------------------------------------------------------
c First try and run a 6x6 spline interpolation
c Next  try and run a 4x4 spline interpolation
c If close to an edge, run bilinear interpolation
c ----------------------------------------------------------
      if     ( irown.ge.3 .and. irown.lt.(nla(kk)-2)    .and.
     +         icoln.ge.3 .and. icoln.lt.(nlo(kk)-2) )  then
        call spline6 (xlat,xlon,lin,kk,  val)

      elseif ( irown.ge.2 .and. irown.lt.(nla(kk)-1)   .and.
     +         icoln.ge.2 .and. icoln.lt.(nlo(kk)-1) ) then
        call spline4 (xlat,xlon,lin,kk, val)
      else
        call bilin (xlat,xlon,lin,kk, val)
      endif

      return
      end
