C /ngslib/source/Geoid/Flib/SCCS/s.spline.f

      real*8 function spline (X, YY, NN, RR)
********************************************************************************
* FAST ONE-DIMENSIONAL EQUIDISTANT SPLINE INTERPOLATION FUNCTION.
* REFERENCE: JOSEF STOER: EINFUHRUNG IN DIE NUMERISCHE MATHEMATIK, I
* SPRINGER 1972, PAGE 81.
*    in - X  : INTERPOLATION ARGUMENT (REAL)
*              X = 1 FIRST DATA-POINT,  X = NN LAST DATA-POINT.
*              OUTSIDE THE RANGE LINEAR EXTRA-POLATION IS USED.
*
*    in - YY : REAL*8 ARRAY, 1 .. NN : DATA VALUES
*
*    in - RR : DO: SPLINE MOMENTS CALCULATED BY SUBROUTINE 'INITSP'
* --------------------------------------------------------------------
* 1983 Jun    Original by RENE FORSBERG
* 2007 May    Modified at NGS
* --------------------------------------------------------------------
********************************************************************************
      implicit none
      character*80 SCCSID
      real*4  X
      real*8  YY
      real*8  RR
      integer NN
      integer JJ
      real*4  XX
      integer DEBUG
      parameter (DEBUG=0)

      DIMENSION YY(NN), RR(NN)

      SCCSID="@(#)spline.f	1.1	2008/04/23 NGS"

C Lower end extrapolation
      if (X.lt. 1)  then
cc        print *, ' Lower end extrapolation'
        spline = YY(1) + (X-1) * (YY(2) - YY(1) - RR(2)/6)

C Upper end extrapolation
      elseif (X.gt.NN)  then
cc        print *, ' Upper end extrapolation'
        spline = YY(NN) + (X-NN) * (YY(NN) - YY(NN-1) + RR(NN-1)/6)

C Interpolation
      else
cc        print *, ' Spline interpolation'
        JJ  = int(X)                !  +1 check this billw
        XX = X - JJ
        if (DEBUG .gt.2) then
          print *, 'XX      = ',XX
          print *, 'JJ      = ',JJ
          print *, 'YY(JJ)  = ',YY(JJ)
        endif
        spline = YY(JJ) + XX * ((YY(JJ+1)-YY(JJ)-RR(JJ)/3-RR(JJ+1)/6) 
     &                  + XX * (RR(JJ)/2 
     &                  + XX * (RR(JJ+1)-RR(JJ))/6))
      endif

      RETURN
      END

