C /ngslib/source/Geoid/Flib/SCCS/s.which1.f

      subroutine which1(xlat,xlon,nfiles,kk,imodel)
c --------------------------------------------------------------------
c - Subroutine to decide which of the open
c - geoid files will be used to interpolate
c - to a point at xlat/xlon.  The returned
c - value of "kk" means the "kkth" file will
c - be used.
c - 
c - Exception:
c - For the GEOID09 geoid model: 
c      Alaska (gCCYYa04.bin) and CONUS (gCCYYu01.bin) overlap. 
c -    This code forces a "Alaska wins" scenario when 
c -    interpolating the geoid at points in the overlap region.
c -    There is sparcity of benchmarks with GPS observations in Alaska, 
c -    and hence, sparcity of orthometric and ellipsoid height data)
c -    However, the Alaska model file is used to avoid abrupt change at edges.
c --------------------------------------------------------------------
      character*80 SCCSID

      real*8     xlat
      real*8     xlon
      integer*4  nfiles
      integer*4  kk
      integer*4  imodel
      integer*4  ii

      real*8    glamn(50), glamx(50), glomn(50), glomx(50)
      real*8    dla(50), dlo(50)
      integer*4 nla(50), nlo(50), ikind(50)
      integer*4 ios(50)
      integer*4 rank(50)

      logical ne,se,we,ee

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Use common statements to cut down on the
c - amount of data sent through 'call' statements
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      common /grid1/ glamn,glamx,glomn,glomx
      common /grid2/ dla,dlo,nla,nlo,ikind 
      common /grid3/ ios 
      common /edges/ ne,se,we,ee

      SCCSID="@(#)which1.f	1.7	2009/09/23 NGS"


c --------------------------------------------------------------------
c This is the Alaska/CONUS overlap exception code
c In the overlap region, then use Alaska
c no matter how close to an edge.
c 
c       Alaska Grid 4     |234E
c                         |
c                |--------|------------58N
c                |        |
c                |        | 
c                |        |
c        49N-----|--------|
c                |              CONUS grid 1
c            230E|
c
c     imodel  = 7  (Geoid09)
c     ii      = 1  (gCCYYu01.bin = the CONUS  grid file)
c     ii      = 12 (gCCYYa04.bin = the Alaska grid file)
c     ios(12) = 0  (Alaska #4 grid file open ok)
c --------------------------------------------------------------------
      if(imodel.eq.7 .and. ios(12) .eq. 0) then
        if( xlat.ge.49  .and.   xlat.le.58  .and. 
     *      xlon.le.234 .and.   xlon.ge.230) then
          kk = 12
          return
        endif
      endif


c --------------------------------------------------------------------
c - Spin through all *open* files, and
c - *RANK* them...the file with the
c - highest RANK decides the value of "kk".
c - Here's how the ranking goes:
c -   0 = Point does not lie in this file area
c -   1 = Point lies in this file, but at a corner (bilinear)
c -   2 = Point lies in this file, but at an edge  (bilinear)
c -   3 = Point lies in this file, 1+ away from corners/edges (4x4 spline)
c -   4 = Point lies in this file, 2+ away from corners/edges (6x6 spline)
c - If a rank=4 file is found, kk is immediately set and we return.
c - If no 4 files are found, 
c - the file with the highest rank (3 or 2 or 1) sets kk.
c - If all files have rank=0, kk is set to -1
c --------------------------------------------------------------------
      do 1 ii=1,nfiles
        if(ios(ii).ne.0)goto 1

        rank(ii) = 0

        if(xlat.le.glamx(ii) .and. xlat.ge.glamn(ii) .and.
     *     xlon.le.glomx(ii) .and. xlon.ge.glomn(ii) ) then

c - At this point, we are inside a grid

c - 6x6 spline test
          if( (xlat-glamn(ii)) .gt. (2.0*dla(ii)) .and.
     +        (glamx(ii)-xlat) .gt. (2.0*dla(ii)) .and.
     +        (xlon-glomn(ii)) .gt. (2.0*dla(ii)) .and.
     +        (glomx(ii)-xlon) .gt. (2.0*dla(ii)) ) then
            rank(ii) = 4
            kk = ii
            return

c - 4x4 spline test
          else if( (xlat-glamn(ii)) .gt. dla(ii) .and.
     +             (glamx(ii)-xlat) .gt. dla(ii) .and.
     +             (xlon-glomn(ii)) .gt. dla(ii) .and.
     +             (glomx(ii)-xlon) .gt. dla(ii) ) then
            rank(ii) = 3
          else

c - Edge and corner test
c - Find which if any of the 9 possible edge conditions exist
c -   NW corner, North Edge, NE corner
c -   West Edge, Center    , East edge
c -   SW corner, South Edge, SE corner
            ne = .false.
            se = .false.
            we = .false.
            ee = .false.

c - Near North edge?
            if( (glamx(ii) - xlat) .le. dla(ii) ) ne=.true.
c - Near South edge?
            if( (xlat - glamn(ii)) .le. dla(ii) ) se=.true.
c - Near East edge?
            if( (glomx(ii) - xlon) .le. dlo(ii) ) ee=.true.
c - Near West edge?
            if( (xlon - glomn(ii)) .le. dlo(ii) ) we=.true.

c - Set the rank of this file, based on edge-logic
            if(ne .and. .not.we .and. .not.ee)  rank(ii) = 2
            if(se .and. .not.we .and. .not.ee)  rank(ii) = 2
            if(we .and. .not.ne .and. .not.se)  rank(ii) = 2
            if(ee .and. .not.ne .and. .not.se)  rank(ii) = 2
            if(ne.and.we)  rank(ii) = 1
            if(se.and.we)  rank(ii) = 1
            if(se.and.ee)  rank(ii) = 1
            if(ne.and.ee)  rank(ii) = 1
          endif
        endif
    1 continue

c --------------------------------------------------------------------
c - If we reach this point, all possible files have been searched, 
c - and there's no open file which had a rank of 4 (6x6 spline). 
c - So now, see if we have any rank 3, 2 or 1 files to use.
c --------------------------------------------------------------------
      do 103 ii=1,nfiles
        if(ios(ii).ne.0) goto 103
        if(rank(ii).eq.3)then
          kk = ii
          return
        endif
  103 continue

      do 102 ii=1,nfiles
        if(ios(ii).ne.0) goto 102
        if(rank(ii).eq.2) then
          kk = ii
          return
        endif
  102 continue

      do 101 ii=1,nfiles
        if(ios(ii).ne.0) goto 101
        if(rank(ii).eq.1) then
          kk = ii
          return
        endif
  101 continue

c - If we come here, no files are acceptable for the
c - given lat/lon value
      kk = -1 
      return
      end

